;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================
;Dungeons by Horizontal flip.
;by J Chalmers
;from Your Computer August, 1982 page 74.
;================================

@KEYBOARD=02BB
@DECODE=07BD
@POSITION=4032
@PRINTAT=08F5
@MOVES=4021
@OUTNUM=0A98
@BOMBS=407B
@TRIES=407C


ORG 4082
;==============================================================================
;When the program is run, first the maze is set up. The random values for each 
;room are chosen. This is done using the Basic RND function. The values obtain-
;ed are Poked into RAM between 16514 and 16614, the area reserved for the maze.
;==============================================================================
	00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
	00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
	00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
	00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
	00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
	00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
	00 00 00 00
;==============================================================================
;PERIMETER, 16614 (40E6)
;When all the values have been stored, the program jumps into machine code. The 
;first routine to be executed is called "perimeter", and makes sure that a wall 
;exists all around the maze so that the player keeps within its confines.
;==============================================================================
@LB40E6
	LD B,0A 
	LD HL,4082 
@LB0001
	SET 3,(HL)		;Set top walls 
	INC HL 
	DJNZ Lb0001
	LD B,0A 
	LD DE,000A 
	LD HL,4082 
@LB0002
	SET 1,(HL)		;Set left walls 
	ADD HL,DE 
	DJNZ Lb0002
	LD B,0A 
	LD HL,408B 
@LB0003
	SET 0,(HL) 		;Set right walls
	ADD HL,DE 
	DJNZ Lb0003
	LD B,0A 
	LD HL,40DC 
@LB0004
	SET 2,(HL) 		;Set bottom walls
	INC HL 
	DJNZ Lb0004
;==============================================================================
;LEGALISE, 16657 (4111)
;The maze producer. Its job is to look at each room in turn, and make sure that
;if a player goes, say, through a door on the right, he comes into the next 
;room through a door on the left. When this routine has been completed, the 
;maze is set up and ready to be wandered through.
;==============================================================================
	LD B,64 
	LD HL,4082 
@LB0009
	BIT 0,(HL) 		;Door on right?
	CALL Z RIGHT
	BIT 1,(HL) 		;Door on left?
	CALL Z LEFT
	BIT 2,(HL) 		;Door down?
	CALL Z DOWN
	BIT 3,(HL) 		;Door up?
	CALL Z UP
	INC HL 			;Next room
	DJNZ Lb0009
	RET
	NOP 
@RIGHT
	LD DE,0001 
	ADD HL,DE 
	RES 1,(HL) 
	JR Lb0010
@LEFT
	LD DE,FFFF 
	ADD HL,DE 
	RES 0,(HL) 
	JR Lb0010
@DOWN
	LD DE,000A 
	ADD HL,DE 
	RES 3,(HL) 
	JR Lb0010
@UP
	LD DE,FFF6 
	ADD HL,DE 
	RES 2,(HL) 
@LB0010
	XOR A 
	SBC HL,DE 
	RET
;==============================================================================
;ROOM DATA
;The data needed to print the walls and doors which make up a room, is stored 
;between 4151 and 41AE.
;==============================================================================
@LB4151
	07 03 03 84 00 00 07 03 03 84 1B		;Up door
@LB415C 
	07 03 03 03 03 03 03 03 03 84 1B 		;Up wall
@LB4167
	82 83 83 81 00 00 82 83 83 81 1B 		;Down door
@LB4172
	82 83 83 83 83 83 83 83 83 81 1B		;Down wall 
@LB417D
	05 76 05 76 82 76 00 76 00 76 07 76 05 76 05 1B ;Door
@LB418D
	05 76 05 76 05 76 05 76 05 76 05 76 05 76 05 1B ;Wall
@LB419D
	85 85 81 00 00 84 85 85 1B 			;Right door
@LB41A6
	85 85 85 85 85 85 85 85 1B 			;Right wall
@LB41AF
	B2 B4 BB AA B8 1B 				;MOVES
@LB41B5
	A7 B4 B2 A7 B8 1B				;BOMBS
@LB41BB
	B9 B7 AE AA B8 1B				;TRIES
;==========================
;PRINT DATA, 16833 (41C1)
;==========================
@LB0011
	LD A,(HL) 
	CP "." 			;EOL Marker
	RET Z 
	RST 10			;Print A
	INC HL 			;Next chr
	JR Lb0011
;==========================
;GET KEY, 16841 (41C9)
;==========================
@LB0012
	CALL KEYBOARD		;Call keyscan
	LD A,FF 
	CP L 
	JR NZ Lb0012		;Wait for key release
@LB0013
	CALL KEYBOARD		;Call leyscan
	LD B,H 
	LD C,L 
	LD D,C 
	INC D 
	JR Z Lb0013
	CALL DECODE		;Call key change
	LD A,(HL) 		;Load A with key
	RET
;==============================================================================
;PRINT ROOM, 16863 (41DF)
;The "print-room" routine prints a plan view of the room which the player is 
;in. The variable which holds the address of the player's position, is the 
;system variable Seed at 4032. The room that the player starts in is chosen 
;randomly by line 80 of the Basic program, which uses Rand to store the 
;selected address in Seed.
;==============================================================================
@PRINTROOM
	LD HL,(POSITION)	;Get Player's position
	PUSH HL 
	BIT 3,(HL)
	LD HL, Lb4151 		;Up Door data
	JR Z Lb0014		;Jump if no door
	LD HL, Lb415C 		;Up Wall data
@LB0014
	PUSH HL 
	LD BC,0100 		;Set print position
	CALL PRINTAT		;Call PRINT-AT
	POP HL 
	CALL Lb0011		;Print data
	POP HL 
	PUSH HL 
	BIT 2,(HL)
	LD HL, Lb4167 		;Down door data
	JR Z Lb0015		;Jump if no door
	LD HL, Lb4172 		;Down wall data
@LB0015
	PUSH HL 
	LD BC,0A00  		;Set print position
	CALL PRINTAT		;Call PRINT-AT
	POP HL 
	CALL Lb0011		;Print data
	POP HL 
	PUSH HL 
	BIT 0,(HL) 
	LD HL, Lb419D 		;Right door data
	JR Z Lb0016		;Jump if no door
	LD HL, Lb41A6 		;Right wall data
@LB0016
	PUSH HL 
	LD BC,0200  		;Set print position
	CALL PRINTAT		;Call PRINT-AT
	POP HL 
@LB4223
	LD A,(HL)		;Print right hand wall or door data
	CP "." 
	JR Z Lb0017
	LD B,09 
@LB0018
	XOR A 
	RST 10			;Print space
	DJNZ Lb0018
	LD A,(HL) 
	RST 10			;Print A
	LD A,76 
	RST 10			;Newline
	INC HL 
	JP Lb4223
@LB0017
	POP HL 
	BIT 1,(HL) 
	LD HL, Lb417D		;Door data 
	JR Z Lb0019		;Jump if no door
	LD HL, Lb418D		;Wall data 		
@LB0019
	PUSH HL 
	LD BC,0200 		;Set print position
	CALL PRINTAT		;Call PRINT-AT
	POP HL 
	CALL Lb0011		;Print data
	RET
;==========================
;PRINT SCORES, 16974 (424E)
;==========================
@LB424E
	LD BC,0F00 		;Set print position
	CALL PRINTAT		;Call PRINT-AT
	LD HL, Lb41AF 		;MOVES data
	CALL Lb0011		;Print "MOVES"
	LD B,00 
	LD A,(MOVES)		;Moves made
	LD C,A 			;BC=Moves made
	XOR A 
	RST 10			;Print space
	CALL OUTNUM		;Print BC, score as a decimal
	LD A,76
	RST 10			;Newline
	RST 10			;Newline
	LD HL, Lb41B5 		;BOMBS data
	CALL Lb0011		;Print "BOMBS"
	LD B,00 
	LD A,(BOMBS)		;Bombs left
	LD C,A 			
	XOR A
	RST 10			;Print space
	CALL OUTNUM		;Print BC, bombs left as decimal
	LD A,76
	RST 10			;Newline
	RST 10			;Newline
	LD HL, Lb41BB 		;TRIES data
	CALL Lb0011		;Print "TRIES"
	LD B,00 
	LD A,(TRIES)		;Tries made
	LD C,A
	XOR A 			
	RST 10			;Print space
	CALL OUTNUM		;Print BC, tries made as decimal
	RET
;==========================
;PERFORM KPRESS 17040 (4290)
;==========================
@LB4290
	CALL KPRESS
	LD (POSITION),HL	;Store Player's position
	RET
;==============================================================================
;A player can move up, down, to the left, or to the right, using the arrowed 
;cursor keys, 5, 6, 7, and 8, providing there is a door to move through. If 
;there is no door, he has the option of using force and can blow a hole into a 
;wall using a bomb. He does this by first pressing the A key and then the arrow
;key indicating the direction of blast required.
;==============================================================================
@KPRESS
	CALL Lb0012
	LD BC,000A 
	LD HL,(POSITION)	;Get Player's position
	CP "5" 
	JR NZ KEY6
	BIT 1,(HL)		;Move left 
	JR NZ KEY6		;IF NZ No left door
	DEC HL 
	RET
;==========================
@KEY6
	CP "6"
	JR NZ KEY7
	BIT 2,(HL) 		;Move down
	JR NZ KEY7		;IF NZ No down door
	ADD HL,BC 
	RET
;==========================
@KEY7
	CP "7"			
	JR NZ KEY8
	BIT 3,(HL) 		;Move up
	JR NZ KEY8		;IF NZ No up door
	SBC HL,BC 
	RET
;==========================
@KEY8
	CP "8"
	JR NZ KEYA
	BIT 0,(HL) 
	JR NZ KEYA		;IF NZ No right door
	INC HL 
	RET
;==========================
;Set BOMB routine
;==========================
@KEYA
	CP "A"
	JR NZ TRYS		;IF NZ no bomb drop
	LD A,(BOMBS) 		;Drop bomb
	CP 00 
	RET Z 			;IF no bombs RETURN
	DEC A 
	LD (BOMBS),A		;Decrease bomb count
@BOMBLOOP
	CALL Lb0012		;Wait for key release
	LD HL,(POSITION)	;Get Player's position
	CP "5"
	JR NZ BOMB6
	RES 1,(HL) 
	JR Lb0026
@BOMB6
	CP "6"
	JR NZ BOMB7
	RES 2,(HL) 
@LB0026
	JR Lb0028
@BOMB7
	CP "7"
	JR NZ BOMB7
	RES 3,(HL) 
@LB0028
	JR Lb0030
@BOMB8
	CP "8"
	JR NZ BOMBLOOP
	RES 0,(HL) 
@LB0030
	PUSH HL 
	CALL Lb40E6
	POP HL 
	RET
;==========================
;Tries
;==========================
@TRYS
	CP 76 
	JP NZ KPRESS
	PUSH HL 
	LD HL,TRIES 
	INC (HL) 
	POP HL 
	LD A,82 
	CP L 
	RET NZ 
	LD A,40 
	CP H 
	RET NZ 
	POP BC 
	POP BC 
	RET
	;******************************************
	;Add here your own routines
;==========================
;COMMAND ROUTINE, 17175, (4317)
;==========================
@LB4317
	CALL Lb40E6			;Perimeter
@LB0031
	CALL PRINTROOM			;Print room
	CALL Lb424E			;Print scores
	;******************************************
	;Here you add the calls to your own routines
	CALL Lb4290			;Keypress
	LD HL,4021 
	INC (HL) 
	JR Lb0031
